#
# See the file "LICENSE" for the full license governing this code.
#
require 'easy_type'

Puppet::Type.type(:db_listener).provide(:db_listener) do
  mk_resource_methods

  def self.instances
    []
  end

  def listener_control(action)
    oracle_base  = resource[:oracle_base_dir]
    listenername = resource[:listener_name]

    if action == :start
      oracle_home     = resource[:oracle_home_dir]
      user            = resource[:os_user]
      listener_action = "start #{listenername}"
    else
      oracle_home     = oracle_home_dir
      user            = os_user
      listener_action = "stop #{listenername}"
    end

    if Facter.value(:kernel) == 'windows'
      command = "#{oracle_home}/bin/lsnrctl.exe #{listener_action}"
      output = `command`
    else
      command = "#{oracle_home}/bin/lsnrctl #{listener_action}"
      full_command = "export ORACLE_HOME=#{oracle_home};export ORACLE_BASE=#{oracle_base};export LD_LIBRARY_PATH=#{oracle_home}/lib;cd #{oracle_home};#{command}"
      options = { :failonfail => true, :combine => true, :uid => user }
      Puppet.info "listener action: #{action} with command #{command}"
      output = Puppet::Util::Execution.execute(full_command, options)
    end

    Puppet.info "listener result: #{output}"
  end

  def listener_processes
    return @listeners if @listeners
    kernel = Facter.value(:kernel)

    if kernel == 'windows'
      # sc query type=service | findstr /B /E /N /R /C:"SERVICE_NAME: OracleTNSListener"
      if ( listenername == 'listener' || listenername == 'LISTENER' )
        lsnr = ''
      else
        lsnr = listenername
      end
      if Facter.value(:ora_install_homes)
        ora_install_homes_fact = Facter.value(:ora_install_homes)
        oracle_home_name = ora_install_homes_fact[oracle_home]['name']
        service = "Oracle#{oracle_home_name}TNSListener#{lsnr}"
        command = "sc query #{service} | findstr \"STATE.*RUNNING\""

        Puppet.debug "listener_status #{command}"
        output = `#{command}`

        if output != ''
          return 'Found'
        else
          return 'NotFound'
        end
      else
        return 'NotFound'
      end
    else
      ps_bin = (kernel != 'SunOS' || (kernel == 'SunOS' && Facter.value(:kernelrelease) == '5.11')) ? '/bin/ps' : '/usr/ucb/ps'
      ps_arg = kernel == 'SunOS' ? 'awwx' : '-ef'

      command = "#{ps_bin} #{ps_arg} | /bin/grep -v grep | /bin/grep /bin/tnslsnr"

      @listeners = []
      Puppet.debug "listener_status #{command}"
      output = `#{command}`
      Puppet.debug output
      list = output.scan(/^(\S*?)\s+\S+\s+\S+\s+\S+\s+\S+\s+\S+\s+\S+\s+(\S+)\s+(\S+)/)
      list.each do |entry|
        @listeners << { :os_user => entry[0], :oracle_home_dir => entry[1].gsub('/bin/tnslsnr', ''), :name => entry[2] }
      end
      @listeners
    end
  end

  def start
    listener_control :start
  end

  def stop
    listener_control :stop
  end

  def restart
    listener_control :stop
    listener_control :start
  end

  def os_user
    current_listener && current_listener[:os_user]
  end

  def oracle_home_dir
    current_listener && current_listener[:oracle_home_dir]
  end

  def current_listener
    listener_processes.select {|e| e[:name].upcase == resource[:listener_name].upcase}.first
  end

  def ensure
    listener_processes.any? {|e| e[:name].upcase == resource[:listener_name].upcase} ? :start : :stop
  end

  def status
    # name = resource[:instance_name]

    kernel = Facter.value(:kernel)

    ps_bin = (kernel != 'SunOS' || (kernel == 'SunOS' && Facter.value(:kernelrelease) == '5.11')) ? '/bin/ps' : '/usr/ucb/ps'
    ps_arg = kernel == 'SunOS' ? 'awwx' : '-ef'

    oracle_home = resource[:oracle_home_dir]
    listenername = resource[:listener_name]

    if Puppet::Util::Platform.windows?
      # TODO Add ORACLE_HOME_NAME in the findstr: findstr /i \"tnslsnr.exe.*Oracle.#{ORACLE_HOME_NAME}TNSListener\""
      command = "tasklist /svc | findstr /i \"tnslsnr.exe.*Oracle.*TNSListener\""
    else
      command = "#{ps_bin} #{ps_arg} | /bin/grep -v grep | /bin/grep -i #{oracle_home}/bin/tnslsnr.*#{listenername}"
    end

    Puppet.debug "listener_status #{command}"
    output = Puppet::Util::Execution.execute(command, { :failonfail => false, :combine => true })
    if Puppet::Util::Platform.windows?
      # TODO same as above
      output.scan(/tnslsnr.exe.*Oracle.*TNSListener/).empty? ? :stop : :start
    else
      output.scan(/#{oracle_home}\/bin\/tnslsnr.*#{listenername}/i).empty? ? :stop : :start
    end
  end
end
