#
# See the file "LICENSE" for the full license governing this code.
#
require 'easy_type'
require 'puppet_x/enterprisemodules/ora_install/get_homes'
require 'puppet_x/enterprisemodules/ora_install/commands'

Puppet::Type.type(:ora_opatch).provide(:base) do
  include EasyType::Provider
  extend ::PuppetX::EnterpriseModules::OraInstall::GetHomes
  include PuppetX::EnterpriseModules::OraInstall::Commands

  mk_resource_methods

  def os_user
    @os_user ||= self.class.user_for_file(oracle_product_home_dir)
  end

  desc 'Base provider for Oracle patches'

  mk_resource_methods

  confine :true => false # This is NEVER a valid provider. It is just used as a base class

  def self.prefetch(resources)
    orainst_dir = check_single_orainst_dir(resources)
    all_patches = oracle_homes(resources).collect { |h| patches_in_home(h, orainst_dir).collect { |p| p[:name] } }.flatten
    resources.keys.each do |patch_name|
      if all_patches.include?(patch_name)
        resources[patch_name].provider = new(:name => name, :ensure => :present)
      end
    end
  end

  [:patch_id, :oracle_product_home_dir, :orainst_dir, :source, :tmp_dir].each do |prop|
    define_method(prop) do
      resource[prop]
    end
  end

  def remove_incompatible_patches
    fact = Facter.value(:ora_install_homes) || {}
    @installed_patches ||= fact.dig(oracle_product_home_dir, 'installed_patches') || []
    resource[:remove_patches_before].each do | remove_patch_id|
      if @installed_patches.include?(remove_patch_id) || force_remove
        Puppet.debug "Incompatible patch #{remove_patch_id} will be removed from #{oracle_product_home_dir}."
        begin
          opatch("rollback -id #{remove_patch_id} -silent -oh #{oracle_product_home_dir} -local", {:os_user => os_user, :oracle_home => oracle_product_home_dir})
        rescue
          raise unless force_remove unless force_remove
          Puppet.debug "uninstall of patch #{remove_patch_id} failed, but we are continuing..."
        end
        #
        # because multiple patches can have the same incompatible patch, make sure
        # it is only removed once.
        #
        @installed_patches.delete(remove_patch_id)  
        #
        # Now see if any of the incompatible patches named in remove_patches_before
        # is already available. If so, we make sure we change the provide so it
        # is no longer installed. The removal of incompatible patches doesn't need
        # the provider. But if you want to re-apply the patch after the removal,
        # the provider must report it absent
        #
        title_name = "#{oracle_product_home_dir}:#{remove_patch_id}"
        catalog_resource = resource.catalog.resource(:ora_opatch, title_name)
        catalog_resource.provider= self.class.new(:name => title_name, :ensure => :absent) if catalog_resource
      else
        Puppet.debug "Incompatible patch #{remove_patch_id} not found in #{oracle_product_home_dir}."
      end
    end
  end


  private

  def self.check_single_orainst_dir(resources)
    orainst_dir = resources.map { |k, v| v.orainst_dir }.uniq
    raise "ora_opatch doesn't support multiple orainst_dir" if orainst_dir.size > 1
    orainst_dir.first
  end
end
