#
# See the file "LICENSE" for the full license governing this code.
#
# rubocop: disable Style/ClassAndModuleCamelCase
module PuppetX
  module EnterpriseModules
    module OraInstall
      # docs
      class Command
        # include Settings

        DEFAULT_TIMEOUT = 300 # 5 minutes

        VALID_OPTIONS = [
          :os_user,
          :oracle_home
        ].freeze

        attr_reader(*VALID_OPTIONS)

        def initialize(command, options, valid_options = VALID_OPTIONS)
          @valid_options = valid_options
          check_options(options)
          @command  = command
        end

        def command_string(arguments = '', command_options)
          if command_options[:oracle_home] && command_options[:os_user]
            oracle_home = command_options[:oracle_home]
            @os_user = command_options[:os_user]
          else
            fail "oracle_home not specfied"              
          end
          "su #{@os_user} -c \"cd #{working_dir}; export ORACLE_HOME=#{oracle_home}; export PATH=$PATH:#{oracle_home}/bin:#{oracle_home}/OPatch; #{@command} #{arguments}\""
        end

        def execute(arguments)
          options = { :failonfail => true, :combine => true }
          value = ''
          Puppet.debug "Executing #{@command} command: #{arguments} on #{@sid} as #{os_user}, connected as #{username}"
          value = Puppet::Util::Execution.execute(command_string(arguments), options)
          value
        end

        protected

        def working_dir
          pwd = Dir.pwd
          unless writable_by_user(@os_user, pwd)
            Puppet.debug "Running Oracle command from a pwd that is not writable for os user #{os_user} therefore resetting pwd to '/tmp'"
            pwd = '/tmp'
          end
          pwd
        end

        private

        def writable_by_user(os_user, pwd)
          system "su - #{os_user} -c \"cd #{pwd};\" > /dev/null 2>&1"
        end

        def within_time(timeout)
          Puppet.debug "Using timeout #{timeout}"
          if timeout.zero?
            yield
          else
            Timeout.timeout(timeout) do
              yield
            end
          end
        end

        def check_options(options)
          options.each_key { |key| raise ArgumentError, "option #{key} invalid for #{@command}. Only #{@valid_options.join(', ')} are supported" unless @valid_options.include?(key) }
        end
      end
    end
  end
end
