#
# ora_install::listener
#
# @summary Manage the oracle listener[DEPRECATED]
# This defined type is deprecated. Please use [db_listener](/docs/ora_install/db_listener.html) custom type to manage your listener.
# 
# Here is an example on how to use this:
# 
# ```puppet
# ora_install::listener{'start listener':
#   action        => 'start',
#   oracle_base   => '/oracle',
#   oracle_home   => '/oracle/product/11.2/db',
#   user          => 'oracle',
#   group         => 'dba',
#   listener_name => 'listener'
# }
# ```
#
# @param [Enum['running', 'start', 'abort', 'stop']] action
#    The action to take.
#    Valid values are:
#    - `running`
#    - `start`
#    - `abort`
#    - `stop`
#    The default value is: `start`
#
# @param [String[1]] group
#    The os group to use for these Oracle puppet definitions.
#    The default value is: `dba`
#
# @param [String[1]] listener_name
#    The name of the listener process.
#    The default value is: `listener`
#
# @param [Stdlib::Absolutepath] oracle_base
#    A directory to use as Oracle base directory.
#
# @param [Stdlib::Absolutepath] oracle_home
#    A directory to be used as Oracle home directory for this software.
#
# @param [String[1]] user
#    The user used for the specified installation.
#    The install class will not create the user for you. You must do that yourself.
#    The default value is: `oracle`
#
#
# See the file "LICENSE" for the full license governing this code.
#
define ora_install::listener(
  Stdlib::Absolutepath                   $oracle_base   = undef,
  Stdlib::Absolutepath                   $oracle_home   = undef,
  String[1]                              $user          = 'oracle',
  String[1]                              $group         = 'dba',
  Enum['running','start','abort','stop'] $action        = 'start',
  String[1]                              $listener_name = 'listener',
)
{
  easy_type::debug_evaluation() # Show local variable on extended debug

  ora_install { $title: ensure => 'present'}

  db_listener{ $title:
    ensure          => $action,
    oracle_base_dir => $oracle_base,
    oracle_home_dir => $oracle_home,
    os_user         => $user,
    listener_name   => $listener_name,
  }
}
